/*************************************************************
This program was produced by the
CodeWizardAVR V1.24.0 Standard
Automatic Program Generator
 Copyright 1998-2003 HP InfoTech s.r.l.
http://www.hpinfotech.ro
e-mail:office@hpinfotech.ro

Project : TP Robot suiveur de ligne "Carte MrLineTiny"
Version : 2
Date    : 10/06/2008
Author  : Philippe Mariano                
Company : LYCEE Pierre Emile MARTIN       
Comments: Suivi de ligne 

Fichier : StayOnLE.c (fichier lve)      

Chip type           : ATtiny26(L)
Clock frequency     : 16,000000 MHz
Memory model        : Tiny
External SRAM size  : 0
Data Stack size     : 32
*************************************************************/

#include <tiny26.h>
#include <delay.h>

// Boolen
//----------------------------------------------------------------------------------------------------- 
#define ACO 	ACSR.5      	// Sortie du comparateur analogique   

// Temporisations
//----------------------------------------------------------------------------------------------------- 
#define t_IR 	19 	// Dure de la commande d'un E_IR (Del IR)
#define t_mux 	540    	// Dure entre deux cde d'un E_IR
#define t_delay 704     // Permet de rgler la priode d'chantillonnage (tech) de la position
			// du robot par rapport  la ligne tech = 3*(t_IR + t_mux) + t_delay
			
// Commande des moteurs  
//----------------------------------------------------------------------------------------------------- 
#define CDE_ROUE_G	OCR1A       //   roue gauche
#define CDE_ROUE_D	OCR1B	    //   roue droite

// Valeurs reprsentatives du rapport cyclique du signal 
// de commande des moteurs (OC1A et OC1B non inverss)
//----------------------------------------------------------------------------------------------------- 
#define _0PC	 255		 // 0%  
#define _10PC	              // 10%
#define _20PC    		 // 20%
#define _30PC    		 // 30% 
#define _40PC    		 // 40%                     //  complter
#define _50PC                 // 50% 
#define _60PC    1             // 60%
#define _70PC                  // 70%
#define _80PC                  // 80%      
#define _90PC                  // 90% 

// Connexions des Dels Visu et IR au port A  
//-----------------------------------------------------------------------------------------------------    						 
// Del_D	relie  PA3 (D1)     E_D_IR	PAO  (DIR1)
// Del_C 	relie  PA4 (D2)     E_C_IR	PA1  (DIR2)
// Del_G	relie  PA5 (D3)     E_G_IR	PA2  (DIR3)

// Mots de commande des Dels Visu   (- Del teinte 	* Del claire)
// ----------------------------------Etat des Dels de visualisation------------------------------------
             					// Del_G | Del_C | Del_D  VisuPosition(16)     Remarque
//-----------------------------------------------------------------------------------------------------
#define Visu_Hors_ligne         0b00111111      // off   | off   | off        3F		- - - 
#define Visu_Droite_Plus        0b00011111      // on    | off   | off 	      1F	        * - -
#define Visu_Centre1		0b00101111      // off   | on    | off        2F		- * -
#define Visu_Droite             0b00001111      // on    | on    | off        0F	        * * -
#define Visu_Gauche_Plus        0b00110111      // off   | off   | on         37		- - * 
#define Visu_Centre2            0b00010111      // on    | off   | on         17	        * - *
#define Visu_Gauche		0b00100111      // off   | on    | on         27         	- * *
#define Visu_Centre3            0b00000111      // on    | on    | on         07		* * *
  
// Prototype des fonctions
//---------------------------
void Arret_Moteurs();
void Active_Timer();

// Variables globales 
//-----------------------------------------------------------------------------------------------------
// type		nom		Commentaires
//----------------------------------------------------------------------------------------------------- 
// Les diffrentes commandes des moteurs
enum Etat_Commande_Moteur {DEPART, CDE_CENTRE1, CDE_CENTRE2, CDE_CENTRE3, 
CDE_DROITE, CDE_DROITE_PLUS, HORS_LIGNE_DROITE, CDE_GAUCHE, CDE_GAUCHE_PLUS, HORS_LIGNE_GAUCHE} 
EtatCdeMot = DEPART;

// Les diffrentes positions du robot par rapport  la ligne 
enum Etat_Position_Robot {HORS_LIGNE, GAUCHE_PLUS , CENTRE_1, GAUCHE, 
DROITE_PLUS, CENTRE_2, DROITE, CENTRE_3} Position_Robot = HORS_LIGNE; 


//----------------------------------------------------------------------------------------------------- 
// Dbut du programme principal
//-----------------------------------------------------------------------------------------------------
void main(void)
{

// Variables locales
//-----------------------------------------------------------------------------------------------------
// type		nom		Commentaires
//----------------------------------------------------------------------------------------------------- 
unsigned char i;                // Compteur de boucles
unsigned char Cde_Del_IR;	// Permet de multiplexer la commande des dels IR
unsigned char VisuPosition;	// Affichage de la position du robot sur 3 dels

// Input/Output Ports initialization
// Port A initialization
// Func0=Out Func1=Out Func2=Out Func3=Out Func4=Out Func5=Out Func6=In Func7=In 
// State0=1 State1=1 State2=1 State3=1 State4=1 State5=1 State6=T State7=T 
PORTA=0x3F;
DDRA=0x3F;

// Port B initialization
// Func0=In Func1=Out Func2=In Func3=Out Func4=In Func5=In Func6=In Func7=In 
// State0=T State1=1 State2=T State3=1 State4=T State5=T State6=P State7=T 
PORTB=0x0A;
DDRB=0x0A;

// Timer/Counter 1 initialization
// Clock source: System Clock
// Clock value: 4000,000 kHz (frquence d'entre du timer FQ/4)
// Mode: PWMA & B top=OCR1C
// OC1A output: Non-Inv., /OC1A disconnected
// OC1B output: Non-Inv., /OC1B disconnected
PLLCSR=0x00;
TCCR1A=0xA3;
TCCR1B=0x03;
TCNT1=0x00;
OCR1A=0x00;
OCR1B=0x00;
OCR1C=0xFF;

// Analog Comparator initialization
// Analog Comparator: On
ACSR=0x00;
     
PORTA = 0x3F;   // Extinction des Dels

while (1)
    {// dbut while
      Position_Robot = HORS_LIGNE;   // Initialisation	
      Cde_Del_IR = 1; 
    
/*----------------------------------------------------------------------------------------------------- 
// Dbut de la tche "Acqurir la position du robot par rapport  la ligne" (Intgre dans FS21 et FS35)
//-----------------------------------------------------------------------------------------------------
FS21 " Commander un des trois metteurs IR(Del)"

Rle : Slectionner un des trois metteurs (Del IR) devant gnrer un faisceau IR
       
Entre : 	Cde_Del_IR 		octet permettant de multiplexer la commande des dels IR     

Sorties:        PA0, PA1, PA2 		bits de commande des dels IR (dtails ci-dessous)
					Emmetteur IR connect  au bit      Remarque 
					----------------------------------------------------         
					E_D_IR		"      PA0    |   Emmeteur IR droit       
					E_C_IR		"      PA1    |	  Emmeteur IR centre      
					E_G_IR 		"      PA2    |	  Emmeteur IR gauche     
//-----------------------------------------------------------------------------------------------------                                                                                       
FS35 "Calculer la position du robot par rapport  la ligne"   

Rle : Etablir une valeur numrique reprsentative de la position du robot par rapport  la ligne
 
Entres : 	AC0		 	bit associ  la sortie du comparateur analogique, 
					reprsentatif de la position d'un capteur sur la ligne 
					
Sortie :	Position_Robot		octet image de la position du robot par rapport  la ligne)
                                        (dtails ci-dessous)
                                        
Position des capteurs IR  =>  "Position_Robot"	// Positions du robot par rapport  la ligne
//-----------------------------------------------------------------------------------------------------
IR_G  | IR_C  |  IR_D    =>     Position_Robot(10)
non  |  non  |   non     =>	   0		// Hors ligne
non  |  non  |   oui     =>	   1         	// Gauche+
non  |  oui  |   non     =>	   2         	// Centre1 (ligne noire de largeur gale  celle d'un capteur)
non  |  oui  |   oui     =>	   3         	// Gauche
oui  |  non  |   non     =>	   4		// Droite+
oui  |  non  |   oui     =>	   5         	// Centre2 (ligne blanche entoure de deux noires)
oui  |  oui  |   non     =>	   6		// Droite
oui  |  oui  |   oui     =>	   7         	// Centre3 (ligne noire de largeur gale  celle de trois capteurs)
    			     	
-----------------------------------------------------------------------------------------------------*/
      for (i=0;i<3;i++)  
       {
        PORTA = ~Cde_Del_IR; 					      // Les E_IR (Leds IR) sont commands  l'tat bas
      	delay_us(t_IR);						      // Dure de l'tat bas (A ajuster en fonction des capteurs) 
        if (ACO == 1) Position_Robot = Position_Robot | Cde_Del_IR;   // Si le capteur IR_x x=(D,C,G] est sur la ligne
        PORTA = 0x3F;                                                 // le bit correspondant de "Position_Robot est mis  1 
        Cde_Del_IR = Cde_Del_IR << 1 ;                                 
        delay_us(t_mux);					      // Tempo permettant d'attendre l'anulation de la ddp sur
       } 							      // l'entre AIN1 (signal de rception IR) avant une nouvelle 
        							      // mesure (A ajuster en fonction des capteurs)
          
       
// Fin de la tche "Acqurir la position du robot par rapport  la ligne" (Fonctions FS21 et FS35) 
//-----------------------------------------------------------------------------------------------------
         

/*-----------------------------------------------------------------------------------------------------                
Dbut de la tche  "Traiter l'information "Position_Robot" pour visualiser la position du robot par rapport 
 la ligne" (Fonction FP1) 
//-----------------------------------------------------------------------------------------------------
Rle :  lorsque le capteur est sur la ligne, la Del correspondante s'claire

Entre:         Position_Robot      	(voir ci-dessus)

Sorties:	PortA            	octet
 					Del de visualisation connecte au bit
					----------------------------------------
					Del_D       "      		PA3      
					Del_C       "	  		PA4
					Del_G       "	  		PA5 
					Remarque : D pour droite, C pour centre, G pour gauche
-----------------------------------------------------------------------------------------------------*/
    			
      switch (Position_Robot)              	       		// Etat des Dels de visualisation     	
       {                                   	       		// Del_G | Del_C | Del_D 	
        case 0: VisuPosition = Visu_Hors_ligne; break; 		// off   | off   | off 	
        case 1: VisuPosition = Visu_Gauche_Plus; break; 	// off   | off   | on 	
        case 2: VisuPosition = Visu_Centre1; break; 		// off   | on    | off 	
        case 3: VisuPosition = Visu_Gauche; break;     		// off   | on    | on 	
        case 4: VisuPosition = Visu_Droite_Plus; break; 	// on    | off   | off  		
        case 5: VisuPosition = Visu_Centre2; break; 		// on    | off   | on 	
        case 6: VisuPosition = Visu_Droite; break; 		// on    | on    | off  	
        case 7: VisuPosition = Visu_Centre3; break;      	// on    | on    | on  	
       }   
         
// Fin de la tche  "Traiter l'information "Position_Robot" pour visualiser la position du robot par 
// rapport  la ligne" (Fonction FP1)
//-----------------------------------------------------------------------------------------------------
         

       PORTA = VisuPosition;  // Fonction FS61 "Slectionner les dels de visualisation"      

/*-----------------------------------------------------------------------------------------------------
Dbut de la tche  "Traiter l'information "Position_Robot" pour corriger la trajectoire du robot" 
(Fonction FP1) 
//-----------------------------------------------------------------------------------------------------
Rle: Rgler le rapport cyclique des signaux de commande des moteurs droit et gauche pour rgler la 
trajectoire du robot afin qu'il suive la ligne

Entre:    Position_Robot             	(voir ci-dessus)    

Sorties:   CDE_ROUE_G         octet	(OCR1A)rgle le rapport cyclique du signal de commande du moteur 
					associ  la roue Gauch
	   CDE_ROUE_D                   (OCR1B)rgle le rapport cyclique du signal de commande du moteur 
	   associ  la roue Droit
-----------------------------------------------------------------------------------------------------*/      
      switch (EtatCdeMot)              
       { // Dbut switch (EtatCdeMot)                                 
        case DEPART: Arret_Moteurs(); 
        		if ((Position_Robot == CENTRE_1) || (Position_Robot == CENTRE_2) || (Position_Robot == CENTRE_3))
        		    { 
        		      delay_ms(3000);
        		      Active_Timer();
        		      EtatCdeMot = CDE_CENTRE1;
        		    }
        		    else  EtatCdeMot = DEPART;
        break;   
        
        case CDE_CENTRE1: EtatCdeMot = CDE_CENTRE3; 
        break;  
        
        case CDE_CENTRE2: EtatCdeMot = CDE_CENTRE3;   
        
        break;   
        
        case CDE_CENTRE3: CDE_ROUE_G = _60PC; CDE_ROUE_D = _60PC;
                          if (Position_Robot == GAUCHE) EtatCdeMot = CDE_DROITE;
                          	else if (Position_Robot == DROITE) EtatCdeMot = CDE_GAUCHE;
                          		else EtatCdeMot = CDE_CENTRE3;
        break; 
        
        case CDE_DROITE:CDE_ROUE_G = _60PC; CDE_ROUE_D = _50PC; 
                         if (Position_Robot == CENTRE_3) EtatCdeMot = CDE_CENTRE3;
                          	else if (Position_Robot == GAUCHE_PLUS) EtatCdeMot = CDE_DROITE_PLUS;
                          		else EtatCdeMot = CDE_DROITE;
        break;   
        
        case CDE_DROITE_PLUS:CDE_ROUE_G = _60PC; CDE_ROUE_D = _40PC;
        	if (Position_Robot == GAUCHE) EtatCdeMot = CDE_DROITE; 
        		else if (Position_Robot == HORS_LIGNE) EtatCdeMot = HORS_LIGNE_DROITE;
                          		else EtatCdeMot = CDE_DROITE_PLUS;
        break;  
        
        case HORS_LIGNE_DROITE:CDE_ROUE_G = _50PC; CDE_ROUE_D = _20PC;
        	if (Position_Robot == GAUCHE_PLUS) EtatCdeMot = CDE_DROITE_PLUS; 
                          		else EtatCdeMot = HORS_LIGNE_DROITE;
        break; 
        
        case CDE_GAUCHE: CDE_ROUE_G = _50PC; CDE_ROUE_D = _60PC; 
                       if (Position_Robot == CENTRE_3) EtatCdeMot = CDE_CENTRE3;
                          	else if (Position_Robot == DROITE_PLUS) EtatCdeMot = CDE_GAUCHE_PLUS;
                          		else EtatCdeMot = CDE_GAUCHE;
        break;  
        
        case CDE_GAUCHE_PLUS: CDE_ROUE_G = _40PC; CDE_ROUE_D = _60PC; 
                       if (Position_Robot == DROITE) EtatCdeMot = CDE_GAUCHE;
                       		else if (Position_Robot == HORS_LIGNE) EtatCdeMot = HORS_LIGNE_GAUCHE;
                          		else EtatCdeMot = CDE_GAUCHE_PLUS;
        break; 
        
        case c_GAUCHE:CDE_ROUE_G = _20PC; CDE_ROUE_D = _50PC;
        	if (Position_Robot == DROITE_PLUS) EtatCdeMot = CDE_GAUCHE_PLUS; 
                          		else EtatCdeMot = HORS_LIGNE_GAUCHE;   
        break;
        
                          		
        default : EtatCdeMot = CDE_CENTRE3;
         
       }// Fin switch (EtatCdeMot)
         
// Fin de la tche  "Traiter l'information "Position_Robot" pour corriger la trajectoire du robot" 
// (Fonction FP1)       
//-----------------------------------------------------------------------------------------------------  

	delay_us(t_delay); // A ajuster en fonction de la vitesse du robot  
      		
    }// fin while(1);
} // fin prog princ   



//-----------------------------------------------------------------------------------------------------
// Sous-programmes
//-----------------------------------------------------------------------------------------------------
void Arret_Moteurs()
{
      TCCR1A = 0x00; // Arrt PWM 
      TCCR1B = 0x00; 
      PORTB=0x0B;     // Arrt Moteur 
} 

void Active_Timer()
{       // Marche PWM
      	TCCR1A=0xA3;
	TCCR1B=0x03;
} 